// RTC-Treiber   Dr. Weisser 13.10.2016     28.2.2019
//ds3231.h

#ifndef DS3231_H_
#define DS3231_H_

#define MON 1
#define TUE 2
#define WED 3
#define THU 4
#define FRI 5
#define SAT 6
#define SUN 7

#define DS1302  0    // 1: DS1302         0: kein DS1302 
#define DS3231  1    // 1: DS3231/DS1307  0: kein DS3231/DS1307 
#define PCF8563 0    // 1: PCF8563        0: kein PCF8563 
#define RV3028  0    // 1: RV3028         0: kein RV3028 


#if DS1302
// RTC-Pins festlegen
// CLK-Pin DS1302
#define RTC_CLK_PORT        PORTD
#define RTC_CLK_DDR         DDRD
#define RTC_CLK_PIN         PIND
#define RTC_CLK_BIT         7                  // Pin 7 am Port
#define RTC_CLK_OUT()       sbi(DDRD,7)        // CLK auf Ausgabe setzen
#define RTC_CLK_H()         sbi(PORTD,7)       // CLK hi setzen
#define RTC_CLK_L()         cbi(PORTD,7)       // CLK low setzen
#define RTC_CLK_VAL         bit_is_set(PIND,7) // CLK level

// RST/CE-Pin DS1302
#define RTC_RST_PORT        PORTB
#define RTC_RST_DDR         DDRB
#define RTC_RST_PIN         PINB
#define RTC_RST_BIT         0                  // Pin 0 am Port
#define RTC_RST_OUT()       sbi(DDRB,0)        // RST/CE auf Ausgabe setzen
#define RTC_RST_H()         sbi(PORTB,0)       // RST/CE hi setzen
#define RTC_RST_L()         cbi(PORTB,0)       // RST/CE low setzen
#define RTC_RST_VAL         bit_is_set(PINB,0) // RST/CE level

// IO-Pin DS1302
#define RTC_IO_PORT         PORTB
#define RTC_IO_DDR          DDRB
#define RTC_IO_PIN          PINB
#define RTC_IO_BIT          1                  // Pin 1 am Port
#define RTC_IO_OUT()        sbi(DDRB,1)        // IO auf Ausgabe setzen
#define RTC_IO_IN()         cbi(DDRB,1)        // IO auf Eingabe setzen
#define RTC_IO_H()          sbi(PORTB,1)       // IO hi setzen
#define RTC_IO_L()          cbi(PORTB,1)       // IO low setzen
#define RTC_IO_VAL          bit_is_set(PINB,1) // IO level: test for high


#else // DS3231/DS1307/PCF8563/RV3028
// defines fuer I2C-Pins stehen in i2c.c
// defines fuer I2C-Adressen Read/Write stehen in ds3231.h
#endif // DS1302


#if DS1302
// DS1302: RTC-Register w: schreiben, sonst lesen
#define SEC_W       0x80   // Sekunden 0-59
                           // 1000 0000
                           //  clock
                           //   A4-A0
                           //         write
#define SEC         0x81   // 1000 0001
                           //         read
#define MIN_W       0x82   // Minuten 0-59
#define MIN         0x83
#define HOUR_W      0x84   // Stunden 0-23
#define HOUR        0x85
#define DATE_W      0x86   // Monatstag 0-31
#define DATE        0x87
#define MONTH_W     0x88   // Monat 1-12
#define MONTH       0x89
#define WDAY_W      0x8a   // Wochentag 1-7 Mo-So
#define WDAY        0x8b
#define YEAR_W      0x8c   // Jahr 00-99
#define YEAR        0x8d
#define W_PROTECT   0x8e   // Schreibschutz
#define C_BURST_W   0xbe   // 1011 1110
                           //  clock
                           //   A4-A0=burst
                           //         write
#define C_BURST     0xbf   // 1011 1111
                           //         read

// DS1302: RTC-RAM w: schreiben, r: lesen
#define RAM_START_W 0xc0   // 1100 0000
                           //  RAM    write
                           //   A4-A0=Adresse der 31 RAM-Register-Zellen
#define RAM_START   0xc1   // 1100 0001
                           //  RAM    read
                           //   A4-A0=Adresse der 31 RAM-Register-Zellen
#define RAM_BURST_W 0xfe   // 1111 1110
                           //  RAM    write
                           //   A4-A0=burst
#define RAM_BURST   0xff   // 1111 1111
                           //  RAM    read
                           //   A4-A0=burst
#endif // DS1302

#if DS3231 // DS3231/DS1307
// DS3231/DS1307: I2C-Adressen
//                                  A2
//                                   A1
//                                    A0
//                                     R=1/W=0
#define RTC_ADR_W   0xD0   // 1101 0000 DS3231 Schreiben
#define RTC_ADR_R   0xD1   // 1101 0001 DS3231 Lesen

// DS3231/DS1307: RTC-Register
#define SEC         0x00   // Sekunden 0-59
#define MIN         0x01   // Minuten 0-59
#define HOUR        0x02   // Stunden 0-23
#define WDAY        0x03   // Wochentag 1-7 Mo-So
#define DATE        0x04   // Monatstag 0-31
#define MONTH       0x05   // Monat 1-12
#define YEAR        0x06   // Jahr 00-99

// DS3231: RTC-Register
#define CONTROL     0x0e   // Control-Register
#define STATUS      0x0f   // Control/Status-Register
#define AGING       0x10   // Aging Offset-Register

#define TEMPERAT_H  0x11   // Temperatur-Register hi
#define TEMPERAT_L  0x12   // Temperatur-Register lo

#endif // DS3231/DS1307

#if PCF8563 // PCF8563
// PCF8563: I2C-Adressen
//                                  A2
//                                   A1
//                                    A0
//                                     R=1/W=0
#define RTC_ADR_W   0xA2   // 1010 0010 PCF8563 Schreiben
#define RTC_ADR_R   0xA3   // 1010 0011 PCF8563 Lesen

#define SEC         0x02   // Sekunden 0-59
#define MIN         0x03   // Minuten 0-59
#define HOUR        0x04   // Stunden 0-23
#define DATE        0x05   // Monatstag 0-31
#define WDAY        0x06   // Wochentag 0-6 Mo-So
#define MONTH       0x07   // Monat 1-12
#define YEAR        0x08   // Jahr 00-99

#endif // PCF8563

#if RV3028 // RV3028
// RV3028: I2C-Adressen
//                                  A2
//                                   A1
//                                    A0
//                                     R=1/W=0
#define RTC_ADR_W   0xA4   // 1010 0100 RV3028 Schreiben
#define RTC_ADR_R   0xA5   // 1010 0101 RV3028 Lesen

#define SEC         0x00   // Sekunden 0-59
#define MIN         0x01   // Minuten 0-59
#define HOUR        0x02   // Stunden 0-23
#define WDAY        0x03   // Wochentag 0-6 Mo-So
#define DATE        0x04   // Monatstag 0-31
#define MONTH       0x05   // Monat 1-12
#define YEAR        0x06   // Jahr 00-99

#endif // RV3028


// Funktionsprototypen

void rtc_set_time_date(    // RTC stellen (nach Batterie-Wechsel)
    unsigned char date,    // 0x31 31.12.15 20:17:00 TUE
    unsigned char month,   // 0x12
    unsigned char year,    // 0x15 (2015)
    unsigned char hour,    // 0x20
    unsigned char min,     // 0x17
    unsigned char sec,     // 0x00
    unsigned char wday);   // 0x02

//void rtc_init(void);                // Portpins DS1302 ggf. definiert setzen
void rtc_write_byte(unsigned char);   // RTC-Byte schreiben
unsigned char rtc_read_byte(void);    // RTC-Roh-Byte lesen
unsigned char rtc_read_clock(void);   // RTC-Clock lesen
void rtc_read_burst(void);            // RTC im Burst lesen
void rtc_write(unsigned char,unsigned char); // RTC Byte in Register schreiben
unsigned char rtc_read(unsigned char);       // RTC Byte aus Register lesen
unsigned long rtc_get_fattime(void);  // RTC im Burst lesen und packen fuer SD-Karte
void rtc(unsigned char);              // RTC-Ausgabe Datum/Zeit auf USART/LCD/SRAM
void rtc_test(void);                  // RTC-Test: Ausgabe USART

#endif
